@@ -13,6 +13,13 @@ def get_empirical_sample_composition(score_array):
 
 
 def score_batch_correction(scores, composition, classifier, weights_file):
+    # Apply the smoothing coefficient to shrink the calibration effect in
+    # very skewed distributions
+    composition_skew = utils.specificity(composition)
+    smoothing_coef = 1 - (composition_skew * 0.3)
+    uniform_dist = np.ones(3) / 3
+    composition = (composition * smoothing_coef) + (uniform_dist * (1 - smoothing_coef))
+    # Set the aggregated classifier as the default
     if classifier not in {"marker", "aggregated", "nn"}:
         classifier = "aggregated"
     # Prepare inputs
@@ -305,7 +312,11 @@ def main(input_path, output_path, composition, force_auto, verbose):
         console.log("The uncalibrated classification scores were loaded.")
 
     # Get sample composition
-    if (len("contig_names") < 1_000) and (composition == "auto") and not force_auto:
+    if (
+        (len(score_dict["contig_names"]) + len(score_dict["provirus_names"]) < 1_000)
+        and (composition == "auto")
+        and not force_auto
+    ):
         console.warning(
             "Your sample has less than 1,000 sequences, which does not allow precise "
             "composition estimation. The 'metagenome' preset will be used instead. If "
