@@ -1,5 +1,6 @@
 import itertools
 import sys
+from collections import defaultdict
 
 import numpy as np
 from genomad import sequence, utils
@@ -22,8 +23,10 @@ def flag_sequences(
     class_index,
     min_score,
     max_fdr,
+    min_marker_enrichment,
+    min_hallmarks,
     max_uscg,
-    n_uscg_dict,
+    filters_dict,
     provirus_name_array=None,
     provirus_score_array=None,
 ):
@@ -44,10 +47,17 @@ def flag_sequences(
     added_contigs = set()
     added_proviruses = set()
     for i in reversed(score_array[:, class_index].argsort()):
+        n_uscg, marker_enrichment, n_hallmarks = filters_dict.get(
+            name_array[i], (0, np.zeros(3), (0, 0))
+        )
+        marker_enrichment = marker_enrichment[class_index]
+        n_hallmarks = n_hallmarks[class_index - 1]
         if (
             score_array[i].argmax() == class_index
             and score_array[i, class_index] >= min_score
-            and n_uscg_dict.get(name_array[i], 0) <= max_uscg
+            and marker_enrichment >= min_marker_enrichment
+            and n_hallmarks >= min_hallmarks
+            and n_uscg <= max_uscg
         ):
             if name_array[i] in provirus_name_set:
                 contig_name = name_array[i].rsplit("|", 1)[0]
@@ -75,7 +85,18 @@ def flag_sequences(
     )
 
 
-def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
+def main(
+    input_path,
+    output_path,
+    verbose,
+    min_score,
+    max_fdr,
+    min_plasmid_marker_enrichment,
+    min_virus_marker_enrichment,
+    min_plasmid_hallmarks,
+    min_virus_hallmarks,
+    max_uscg,
+):
     # Create `output_path` if it does not exist
     if not output_path.is_dir():
         output_path.mkdir()
@@ -90,6 +111,10 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
     parameter_dict = {
         "min_score": min_score,
         "max_fdr": max_fdr,
+        "min_plasmid_hallmarks": min_plasmid_hallmarks,
+        "min_virus_hallmarks": min_virus_hallmarks,
+        "min_plasmid_marker_enrichment": min_plasmid_marker_enrichment,
+        "min_virus_marker_enrichment": min_virus_marker_enrichment,
         "max_uscg": max_uscg,
     }
 
@@ -305,6 +330,14 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
         console.log(f"Creating the [green]{outputs.summary_dir}[/green] directory.")
         outputs.summary_dir.mkdir()
 
+    # Write the execution data to `execution_info_output`
+    utils.write_execution_info(
+        "summary",
+        input_path,
+        parameter_dict,
+        outputs.summary_execution_info,
+    )
+
     if selected_classifier == "calibrated_aggregated":
         console.log(
             "Using calibrated scores from [cyan]aggregated-classification[/cyan]."
@@ -338,29 +371,35 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
         )
 
     # Store the number of USCGs, genetic code, and number of genes
-    n_uscg_dict = {}
     n_genes_dict = {}
     genetic_code_dict = {}
+    filters_dict = {}
     if marker_classification_exec:
-        for k, v1, v2, v3 in zip(
+        for k, v1, v2, v3, v4, v5 in zip(
             np.load(outputs.features_npz_output)["contig_names"],
             np.load(outputs.features_npz_output)["contig_n_uscg"],
             np.load(outputs.features_npz_output)["contig_n_genes"],
             np.load(outputs.features_npz_output)["contig_genetic_code"],
+            np.load(outputs.features_npz_output)["contig_marker_enrichment"],
+            np.load(outputs.features_npz_output)["contig_n_hallmarks"],
         ):
-            n_uscg_dict[k] = v1
             n_genes_dict[k] = v2
             genetic_code_dict[k] = v3
+            filters_dict[k] = (v1, v4, v5)
         if include_provirus:
-            for k, v1, v2, v3 in zip(
+            for k, v1, v2, v3, v4, v5 in zip(
                 np.load(outputs.provirus_features_npz_output)["provirus_names"],
                 np.load(outputs.provirus_features_npz_output)["provirus_n_uscg"],
                 np.load(outputs.provirus_features_npz_output)["provirus_n_genes"],
                 np.load(outputs.provirus_features_npz_output)["provirus_genetic_code"],
+                np.load(outputs.provirus_features_npz_output)[
+                    "provirus_marker_enrichment"
+                ],
+                np.load(outputs.provirus_features_npz_output)["provirus_n_hallmarks"],
             ):
-                n_uscg_dict[k] = v1
                 n_genes_dict[k] = v2
                 genetic_code_dict[k] = v3
+                filters_dict[k] = (v1, v4, v5)
 
     # Load the classificaion results:
     score_dict = {}
@@ -389,8 +428,10 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
             1,
             min_score,
             max_fdr,
+            min_plasmid_marker_enrichment,
+            min_plasmid_hallmarks,
             max_uscg,
-            n_uscg_dict,
+            filters_dict,
         )
         virus_name_array, virus_score_array, virus_fdr_array = flag_sequences(
             score_dict["contig_names"],
@@ -398,8 +439,10 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
             2,
             min_score,
             max_fdr,
+            min_virus_marker_enrichment,
+            min_virus_hallmarks,
             max_uscg,
-            n_uscg_dict,
+            filters_dict,
             score_dict["provirus_names"],
             score_dict["provirus_predictions"],
         )
@@ -414,31 +457,31 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
             outputs.summary_virus_sequences_output, "w"
         ) as fout2:
             for seq in sequence.read_fasta(input_path):
-                if seq.id in plasmid_name_set:
-                    length_dict[seq.id] = len(seq)
+                if seq.accession in plasmid_name_set:
+                    length_dict[seq.accession] = len(seq)
                     if seq.has_dtr():
-                        topology_dict[seq.id] = "DTR"
+                        topology_dict[seq.accession] = "DTR"
                     elif seq.has_itr():
-                        topology_dict[seq.id] = "ITR"
+                        topology_dict[seq.accession] = "ITR"
                     else:
-                        topology_dict[seq.id] = "Linear"
+                        topology_dict[seq.accession] = "Linear"
                     fout1.write(str(seq))
-                elif seq.id in virus_name_set:
-                    length_dict[seq.id] = len(seq)
+                elif seq.accession in virus_name_set:
+                    length_dict[seq.accession] = len(seq)
                     if seq.has_dtr():
-                        topology_dict[seq.id] = "DTR"
+                        topology_dict[seq.accession] = "DTR"
                     elif seq.has_itr():
-                        topology_dict[seq.id] = "ITR"
+                        topology_dict[seq.accession] = "ITR"
                     else:
-                        topology_dict[seq.id] = "Linear"
+                        topology_dict[seq.accession] = "Linear"
                     fout2.write(str(seq))
             if include_provirus:
                 for seq in sequence.read_fasta(
                     outputs.find_proviruses_nucleotide_output
                 ):
-                    if seq.id in virus_name_set:
-                        length_dict[seq.id] = len(seq)
-                        topology_dict[seq.id] = "Provirus"
+                    if seq.accession in virus_name_set:
+                        length_dict[seq.accession] = len(seq)
+                        topology_dict[seq.accession] = "Provirus"
                         fout2.write(str(seq))
         console.log(
             "Nucleotide sequences were written to "
@@ -452,15 +495,15 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
                 outputs.summary_virus_proteins_output, "w"
             ) as fout2:
                 for seq in sequence.read_fasta(outputs.annotate_proteins_output):
-                    if seq.id.rsplit("_", 1)[0] in plasmid_name_set:
+                    if seq.accession.rsplit("_", 1)[0] in plasmid_name_set:
                         fout1.write(str(seq))
-                    elif seq.id.rsplit("_", 1)[0] in virus_name_set:
+                    elif seq.accession.rsplit("_", 1)[0] in virus_name_set:
                         fout2.write(str(seq))
                 if include_provirus:
                     for seq in sequence.read_fasta(
                         outputs.find_proviruses_proteins_output
                     ):
-                        if seq.id.rsplit("_", 1)[0] in virus_name_set:
+                        if seq.accession.rsplit("_", 1)[0] in virus_name_set:
                             fout2.write(str(seq))
             console.log(
                 "Protein sequences were written to "
@@ -469,25 +512,29 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
             )
 
         with console.status("Writting gene annotation data."):
+            conjscan_genes_dict = defaultdict(list)
             with open(outputs.summary_plasmid_genes_output, "w") as fout1, open(
                 outputs.summary_virus_genes_output, "w"
             ) as fout2:
                 fout1.write(
                     "gene\tstart\tend\tlength\tstrand\tgc_content\tgenetic_code\trbs_motif\t"
-                    "marker\tevalue\tbitscore\tuscg\ttaxid\ttaxname\tannotation_accessions\t"
-                    "annotation_description\n"
+                    "marker\tevalue\tbitscore\tuscg\tplasmid_hallmark\tvirus_hallmark\ttaxid\t"
+                    "taxname\tannotation_conjscan\tannotation_accessions\tannotation_description\n"
                 )
                 fout2.write(
                     "gene\tstart\tend\tlength\tstrand\tgc_content\tgenetic_code\trbs_motif\t"
-                    "marker\tevalue\tbitscore\tuscg\ttaxid\ttaxname\tannotation_accessions\t"
-                    "annotation_description\n"
+                    "marker\tevalue\tbitscore\tuscg\tplasmid_hallmark\tvirus_hallmark\ttaxid\t"
+                    "taxname\tannotation_conjscan\tannotation_accessions\tannotation_description\n"
                 )
                 for line in utils.read_file(
                     outputs.annotate_genes_output, skip_header=True
                 ):
-                    if line.split("\t")[0].rsplit("_", 1)[0] in plasmid_name_set:
+                    seq_name = line.split("\t")[0].rsplit("_", 1)[0]
+                    if seq_name in plasmid_name_set:
                         fout1.write(line)
-                    elif line.split("\t")[0].rsplit("_", 1)[0] in virus_name_set:
+                        if line.split("\t")[16] != "NA":
+                            conjscan_genes_dict[seq_name].append(line.split("\t")[16])
+                    elif seq_name in virus_name_set:
                         fout2.write(line)
                 if include_provirus:
                     for line in utils.read_file(
@@ -532,7 +579,8 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
         # Write plasmid summary file
         with open(outputs.summary_plasmid_output, "w") as fout:
             fout.write(
-                "seq_name\tlength\tn_genes\tgenetic_code\tplasmid_score\tfdr\ttopology\n"
+                "seq_name\tlength\ttopology\tn_genes\tgenetic_code\tplasmid_score\t"
+                "fdr\tn_hallmarks\tmarker_enrichment\tconjugation_genes\n"
             )
             for seq_name, score, fdr in itertools.zip_longest(
                 plasmid_name_array,
@@ -541,40 +589,62 @@ def main(input_path, output_path, verbose, min_score, max_fdr, max_uscg):
                 fillvalue="NA",
             ):
                 length = length_dict.get(seq_name, "NA")
+                topology = topology_dict.get(seq_name, "NA")
                 n_genes = n_genes_dict.get(seq_name, "NA")
                 genetic_code = genetic_code_dict.get(seq_name, "NA")
-                topology = topology_dict.get(seq_name, "NA")
                 score = f"{score:.4f}"
                 fdr = f"{fdr:.4f}" if not isinstance(fdr, str) else fdr
+                if annotate_exec:
+                    _, marker_enrichment, n_hallmarks = filters_dict.get(
+                        seq_name, (0, np.zeros(3), (0, 0))
+                    )
+                    n_hallmarks = n_hallmarks[0]
+                    marker_enrichment = f"{marker_enrichment[1]:.4f}"
+                    conjugation_genes = conjscan_genes_dict.get(seq_name, [])
+                    if len(conjugation_genes):
+                        conjugation_genes = ";".join(conjugation_genes)
+                    else:
+                        conjugation_genes = "NA"
+                else:
+                    marker_enrichment = "NA"
+                    n_hallmarks = "NA"
+                    conjugation_genes = "NA"
                 fout.write(
-                    f"{seq_name}\t{length}\t{n_genes}\t{genetic_code}\t{score}\t"
-                    f"{fdr}\t{topology}\n"
+                    f"{seq_name}\t{length}\t{topology}\t{n_genes}\t{genetic_code}\t{score}\t"
+                    f"{fdr}\t{n_hallmarks}\t{marker_enrichment}\t{conjugation_genes}\n"
                 )
 
         # Write virus summary file
         with open(outputs.summary_virus_output, "w") as fout:
             fout.write(
-                "seq_name\tlength\tn_genes\tgenetic_code\tvirus_score\tfdr\ttopology\t"
-                "coordinates\ttaxonomy\n"
+                "seq_name\tlength\ttopology\tcoordinates\tn_genes\tgenetic_code\tvirus_score\t"
+                "fdr\tn_hallmarks\tmarker_enrichment\ttaxonomy\n"
             )
             for seq_name, score, fdr in itertools.zip_longest(
                 virus_name_array, virus_score_array, virus_fdr_array, fillvalue="NA"
             ):
                 length = length_dict.get(seq_name, "NA")
-                n_genes = n_genes_dict.get(seq_name, "NA")
-                genetic_code = genetic_code_dict.get(seq_name, "NA")
                 topology = topology_dict.get(seq_name, "NA")
                 coord = provirus_coord_dict.get(seq_name, "NA")
                 coord = "-".join(map(str, coord)) if isinstance(coord, tuple) else coord
-                if annotate_exec:
-                    taxonomy = taxonomy_dict.get(seq_name, "Unclassified")
-                else:
-                    taxonomy = taxonomy_dict.get(seq_name, "NA")
+                n_genes = n_genes_dict.get(seq_name, "NA")
+                genetic_code = genetic_code_dict.get(seq_name, "NA")
                 score = f"{score:.4f}"
                 fdr = f"{fdr:.4f}" if not isinstance(fdr, str) else fdr
+                if annotate_exec:
+                    _, marker_enrichment, n_hallmarks = filters_dict.get(
+                        seq_name, (0, np.zeros(3), (0, 0))
+                    )
+                    n_hallmarks = n_hallmarks[1]
+                    marker_enrichment = f"{marker_enrichment[2]:.4f}"
+                    taxonomy = taxonomy_dict.get(seq_name, "Unclassified")
+                else:
+                    marker_enrichment = "NA"
+                    n_hallmarks = "NA"
+                    taxonomy = "NA"
                 fout.write(
-                    f"{seq_name}\t{length}\t{n_genes}\t{genetic_code}\t{score}\t"
-                    f"{fdr}\t{topology}\t{coord}\t{taxonomy}\n"
+                    f"{seq_name}\t{length}\t{topology}\t{coord}\t{n_genes}\t{genetic_code}\t"
+                    f"{score}\t{fdr}\t{n_hallmarks}\t{marker_enrichment}\t{taxonomy}\n"
                 )
 
         console.log(
